import { FocusMonitor } from '@angular/cdk/a11y';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { EventEmitter, Directive, ChangeDetectorRef, ElementRef, Attribute, Input, Output, ViewChild } from '@angular/core';

let nextId = 0;
/** Change event object emitted by sbb angular checkboxes. */
class SbbCheckboxChange {
    constructor(
    /** The source checkbox of the event. */
    source, 
    /** The new `checked` value of the checkbox. */
    checked) {
        this.source = source;
        this.checked = checked;
    }
}
class SbbCheckboxBase {
    constructor(_changeDetectorRef, _focusMonitor, elementRef, tabIndex, type = 'checkbox') {
        this._changeDetectorRef = _changeDetectorRef;
        this._focusMonitor = _focusMonitor;
        /** Name value will be applied to the input element if present */
        this.name = null;
        this._checked = false;
        this._disabled = false;
        /** Event emitted when the checkbox's `checked` value changes. */
        this.change = new EventEmitter();
        /** Property that describes the status change of a checkbox field */
        this._onChange = (_) => { };
        /** Property that describes an updating of checkbox */
        this._onTouched = () => { };
        this._uniqueId = `sbb-${type}-${++nextId}`;
        this.id = this._uniqueId;
        this.tabIndex = parseInt(tabIndex, 10) || 0;
        this._focusMonitor.monitor(elementRef, true).subscribe((focusOrigin) => {
            if (!focusOrigin) {
                // When a focused element becomes disabled, the browser *immediately* fires a blur event.
                // Angular does not expect events to be raised during change detection, so any state change
                // (such as a form control's 'ng-touched') will cause a changed-after-checked error.
                // See https://github.com/angular/angular/issues/17793. To work around this, we defer
                // telling the form control it has been touched until the next tick.
                Promise.resolve().then(() => {
                    this._onTouched();
                    _changeDetectorRef.markForCheck();
                });
            }
        });
    }
    /** Id for the inner input field. */
    get inputId() {
        return `${this.id || this._uniqueId}-input`;
    }
    /** Whether the checkbox is required. */
    get required() {
        return this._required;
    }
    set required(value) {
        this._required = coerceBooleanProperty(value);
    }
    /** Whether the checkbox is checked. */
    get checked() {
        return this._checked;
    }
    set checked(value) {
        if (value !== this.checked) {
            this._checked = value;
            this._changeDetectorRef.markForCheck();
        }
    }
    /** Whether the checkbox is disabled. */
    get disabled() {
        return this._disabled;
    }
    set disabled(value) {
        const newValue = coerceBooleanProperty(value);
        if (newValue !== this.disabled) {
            this._disabled = newValue;
            this._changeDetectorRef.markForCheck();
        }
    }
    /** Sets the value in input in the checkbox field */
    writeValue(value) {
        this.checked = !!value;
    }
    /** Method that records the change on a checkbox field */
    registerOnChange(fn) {
        this._onChange = fn;
    }
    /** Method that records the touch on a checkbox field */
    registerOnTouched(fn) {
        this._onTouched = fn;
    }
    /** Method that sets disabled a checkbox */
    setDisabledState(disabled) {
        this.disabled = disabled;
    }
    /** Focuses the checkbox. */
    focus() {
        this._focusMonitor.focusVia(this._inputElement, 'keyboard');
    }
    /** Toggles the `checked` state of the checkbox. */
    toggle() {
        this.checked = !this.checked;
    }
    /**
     * Event handler for checkbox input element.
     * Toggles checked state if element is not disabled.
     */
    _onInputClick(event) {
        // We have to stop propagation for click events on the visual hidden input element.
        // By default, when a user clicks on a label element, a generated click event will be
        // dispatched on the associated input element. Since we are using a label element as our
        // root container, the click event on the `checkbox` will be executed twice.
        // The real click event will bubble up, and the generated click event also tries to bubble up.
        // This will lead to multiple click events.
        // Preventing bubbling for the second event will solve that issue.
        event.stopPropagation();
        if (!this.disabled) {
            this.toggle();
            // Emit our custom change event if the native input emitted one.
            // It is important to only emit it, if the native input triggered one, because
            // we don't want to trigger a change event, when the `checked` variable changes for example.
            this._emitChangeEvent();
        }
    }
    _onInteractionEvent(event) {
        // We always have to stop propagation on the change event.
        // Otherwise the change event, from the input element, will bubble up and
        // emit its event object to the `change` output.
        event.stopPropagation();
    }
    _emitChangeEvent() {
        const event = new SbbCheckboxChange(this, this.checked);
        this._onChange(this.checked);
        this.change.emit(event);
    }
}
SbbCheckboxBase.decorators = [
    { type: Directive }
];
SbbCheckboxBase.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: FocusMonitor },
    { type: ElementRef },
    { type: String, decorators: [{ type: Attribute, args: ['tabindex',] }] },
    null
];
SbbCheckboxBase.propDecorators = {
    id: [{ type: Input }],
    value: [{ type: Input }],
    ariaLabel: [{ type: Input, args: ['aria-label',] }],
    ariaLabelledby: [{ type: Input, args: ['aria-labelledby',] }],
    ariaDescribedby: [{ type: Input, args: ['aria-describedby',] }],
    required: [{ type: Input }],
    name: [{ type: Input }],
    checked: [{ type: Input }],
    disabled: [{ type: Input }],
    change: [{ type: Output }],
    _inputElement: [{ type: ViewChild, args: ['input',] }]
};

/**
 * Generated bundle index. Do not edit.
 */

export { SbbCheckboxBase, SbbCheckboxChange };
//# sourceMappingURL=checkbox.js.map
