import { EventEmitter, Directive, ChangeDetectorRef, Output, Input } from '@angular/core';
import { coerceBooleanProperty } from '@angular/cdk/coercion';

class SbbProcessflowBase {
    constructor(_changeDetectorRef) {
        this._changeDetectorRef = _changeDetectorRef;
        /** Event emitted to the change of step in the process flow. */
        this.stepChange = new EventEmitter();
    }
    ngAfterContentInit() {
        if (this.steps && this.steps.length) {
            Promise.resolve().then(() => {
                this.steps.first.active = true;
                this.steps.first.disabled = false;
            });
        }
    }
    nextStep() {
        const activeStepIndex = this._findActiveStepIndex(this.steps.toArray());
        let activatedStep = false;
        if (activeStepIndex < this.steps.length - 1) {
            this.steps.forEach((s, i) => {
                if (i > activeStepIndex && !activatedStep) {
                    s.active = true;
                    s.disabled = false;
                    activatedStep = true;
                }
                else {
                    s.active = false;
                }
            });
            this._changeDetectorRef.markForCheck();
            this.stepChange.emit(this.steps.toArray()[activeStepIndex + 1]);
        }
    }
    prevStep() {
        const activeStepIndex = this._findActiveStepIndex(this.steps.toArray());
        if (activeStepIndex > 0) {
            this.disableStep(activeStepIndex);
            this.changeStep(activeStepIndex - 1);
        }
    }
    _findActiveStepIndex(steps) {
        return steps.findIndex((s) => !!s.active);
    }
    /**
     * Method to change on a step with a click.
     * @param $event Event generated at the click on a step.
     * @param stepIndex Index of a step.
     */
    stepClick($event, stepIndex) {
        $event.preventDefault();
        const currentIndex = this._findActiveStepIndex(this.steps.toArray());
        if (stepIndex < currentIndex) {
            this.changeStep(stepIndex);
        }
    }
    /**
     * Method to change a step in a process flow.
     * @param index Index of the step to change.
     */
    changeStep(index) {
        const step = this.steps.toArray()[index];
        if (step) {
            this.steps.forEach((s, i) => {
                s.active = false;
                if (i > index) {
                    s.disabled = true;
                }
            });
            step.active = true;
            this._changeDetectorRef.markForCheck();
            this.stepChange.emit(step.descriptor);
        }
    }
    /** Method to disable a step in a process flow.
     * @param index Index of the step to disable.
     */
    disableStep(index) {
        const step = this.steps.toArray()[index];
        if (step) {
            step.disabled = true;
            this._changeDetectorRef.markForCheck();
        }
    }
    reset() {
        this.steps.forEach((s) => {
            s.active = false;
            s.disabled = true;
        });
        this.ngAfterContentInit();
        this._changeDetectorRef.markForCheck();
    }
    getActiveStep() {
        if (this.steps && this.steps.length) {
            const activeStepIndex = this._findActiveStepIndex(this.steps.toArray());
            if (activeStepIndex > -1) {
                return this.steps.toArray()[activeStepIndex];
            }
        }
        return null;
    }
}
SbbProcessflowBase.decorators = [
    { type: Directive }
];
SbbProcessflowBase.ctorParameters = () => [
    { type: ChangeDetectorRef }
];
SbbProcessflowBase.propDecorators = {
    stepChange: [{ type: Output }]
};

class SbbProcessflowStepDescriptor {
    constructor() {
        /** Refers to the active status of a step in a process flow. */
        this.active = false;
        /** Refers to the disable status of a step in a process flow.  */
        this.disabled = true;
    }
}
class SbbProcessflowStepBase {
    constructor(_changeDetectorRef) {
        this._changeDetectorRef = _changeDetectorRef;
        /** Refers to the processFlowStep istance. */
        this.descriptor = new SbbProcessflowStepDescriptor();
    }
    /** Title of a step in a process flow. */
    set title(value) {
        this.descriptor.title = value;
        this._changeDetectorRef.markForCheck();
    }
    get title() {
        return this.descriptor.title;
    }
    /** Refers to the active status of a step in a process flow.  */
    set active(value) {
        this.descriptor.active = coerceBooleanProperty(value);
        this._changeDetectorRef.markForCheck();
    }
    get active() {
        return this.descriptor.active;
    }
    /** Refers to the disable status of a step in a process flow.  */
    set disabled(value) {
        this.descriptor.disabled = coerceBooleanProperty(value);
        this._changeDetectorRef.markForCheck();
    }
    get disabled() {
        return this.descriptor.disabled;
    }
}
SbbProcessflowStepBase.decorators = [
    { type: Directive }
];
SbbProcessflowStepBase.ctorParameters = () => [
    { type: ChangeDetectorRef }
];
SbbProcessflowStepBase.propDecorators = {
    title: [{ type: Input }],
    active: [{ type: Input }],
    disabled: [{ type: Input }]
};

/**
 * Generated bundle index. Do not edit.
 */

export { SbbProcessflowBase, SbbProcessflowStepBase, SbbProcessflowStepDescriptor };
//# sourceMappingURL=processflow.js.map
