import { ESCAPE } from '@angular/cdk/keycodes';
import { Overlay, OverlayConfig } from '@angular/cdk/overlay';
import { _getShadowRoot } from '@angular/cdk/platform';
import '@angular/cdk/portal';
import * as i0 from '@angular/core';
import { Injectable, InjectionToken, EventEmitter, Directive, NgZone, ChangeDetectorRef, ElementRef, Input, ContentChild, TemplateRef, ViewChild, Output } from '@angular/core';
import { SbbIconDirective } from '@sbb-esta/angular-core/icon-directive';
import { Subject, merge, of, fromEvent } from 'rxjs';
import { filter, map, take, switchMap } from 'rxjs/operators';

class SbbTooltipRegistryService {
    constructor() {
        this._tooltipActivationSubject = new Subject();
        this.tooltipActivation = this._tooltipActivationSubject.asObservable();
    }
    activate() {
        this._tooltipActivationSubject.next();
    }
}
SbbTooltipRegistryService.ɵprov = i0.ɵɵdefineInjectable({ factory: function SbbTooltipRegistryService_Factory() { return new SbbTooltipRegistryService(); }, token: SbbTooltipRegistryService, providedIn: "root" });
SbbTooltipRegistryService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root',
            },] }
];

/** Injection token that determines the scroll handling while the calendar is open. */
const SBB_TOOLTIP_SCROLL_STRATEGY = new InjectionToken('sbb-tooltip-scroll-strategy');
/** @docs-private */
function SBB_TOOLTIP_SCROLL_STRATEGY_FACTORY(overlay) {
    return () => overlay.scrollStrategies.reposition();
}
/** @docs-private */
const SBB_TOOLTIP_SCROLL_STRATEGY_FACTORY_PROVIDER = {
    provide: SBB_TOOLTIP_SCROLL_STRATEGY,
    deps: [Overlay],
    useFactory: SBB_TOOLTIP_SCROLL_STRATEGY_FACTORY,
};
class SbbTooltipChangeEvent {
    constructor(
    /** Instance of tooltip component. */
    instance, 
    /** States if the tooltip has been opened by a click. */
    isUserInput = false) {
        this.instance = instance;
        this.isUserInput = isUserInput;
    }
}
let tooltipCounter = 1;
class SbbTooltipBase {
    constructor(_overlay, _tooltipRegistry, _document, _zone, _changeDetectorRef, _elementRef, scrollStrategy) {
        this._overlay = _overlay;
        this._tooltipRegistry = _tooltipRegistry;
        this._document = _document;
        this._zone = _zone;
        this._changeDetectorRef = _changeDetectorRef;
        this._elementRef = _elementRef;
        /** Identifier of tooltip. */
        this.id = `sbb-tooltip-id-${tooltipCounter++}`;
        /** Identifier of tooltip content. */
        this.contentId = `sbb-tooltip-content-id-${tooltipCounter++}`;
        /** Sets whether the overlay can be pushed on-screen if it does not fit otherwise. */
        this.overlayWithPush = false;
        /**
         * Sets whether the overlay's position should be locked in after it is
         * positioned initially.
         * When an overlay is locked in, it won't attempt to reposition itself
         * when the position is re-applied (e.g. when the user scrolls away).
         */
        this.overlayWithLockedPosition = true;
        /** Sets a minimum distance the overlay may be positioned to the edge of the viewport. */
        this.overlayViewportMargin = 8;
        /** Open event to a click on tooltip element. */
        this.opened = new EventEmitter();
        /** Close event to a click on tooltip element. */
        this.closed = new EventEmitter();
        this._closeKeyEventStream = new Subject();
        this._panelClass = ['sbb-tooltip-content'];
        this._scrollStrategy = scrollStrategy;
    }
    /**
     * The icon to be used as click target.
     * By default uses question-mark, but the user can use his own icon using the IconDirective.
     */
    set icon(tooltipIcon) {
        this._icon = tooltipIcon;
    }
    get icon() {
        return this._contentIcon || this._icon || this.defaultIcon;
    }
    /** Checks if a tooltip panel exists */
    get overlayAttached() {
        return this.tooltipRef && this.tooltipRef.hasAttached();
    }
    /**
     * A stream of actions that should close the autocomplete panel, including
     * on blur events.
     */
    get panelClosingActions() {
        return merge(this._closeKeyEventStream, this._getOutsideClickStream(), this._tooltipRegistry.tooltipActivation, this.tooltipRef
            ? this.tooltipRef.detachments().pipe(filter(() => this.overlayAttached))
            : of()).pipe(
        // Normalize the output so we return a consistent type.
        map((event) => (event instanceof SbbTooltipChangeEvent ? event : null)));
    }
    ngOnDestroy() {
        if (this.tooltipRef) {
            this.tooltipRef.detach();
            this.tooltipRef.dispose();
            this._closingActionsSubscription.unsubscribe();
        }
    }
    onClick(event) {
        event.stopPropagation();
        // Used to prevent IE11 from forwarding to the input,
        // if sbb-tooltip is placed inside sbb-label.
        event.preventDefault();
        if (this.overlayAttached) {
            this.close(true);
        }
        else {
            this.open(true);
        }
    }
    /**
     * Opens the tooltip
     * @param isUserInput states if the tooltip has been opened by a click
     */
    open(isUserInput = false) {
        if (!this.overlayAttached) {
            this._tooltipRegistry.activate();
            this._createPopup();
            this.tooltipRef.attach(this.tooltipContentPortal);
            this._closingActionsSubscription = this._subscribeToClosingActions();
            this.opened.emit(new SbbTooltipChangeEvent(this, isUserInput));
        }
    }
    /**
     * Closes the tooltip
     * @param isUserInput states if the tooltip has been opened by a click
     */
    close(isUserInput = false) {
        if (this.overlayAttached) {
            this.tooltipRef.detach();
            this.tooltipRef.dispose();
            this._closingActionsSubscription.unsubscribe();
            this.closed.emit(new SbbTooltipChangeEvent(this, isUserInput));
            this._changeDetectorRef.detectChanges();
        }
    }
    /** Create the popup PositionStrategy. */
    _createTooltipPositionStrategy() {
        return this._overlay
            .position()
            .flexibleConnectedTo(this.tooltipTrigger)
            .withTransformOriginOn('.sbb-tooltip-content')
            .withFlexibleDimensions(false)
            .withViewportMargin(8)
            .withPush(false)
            .withPositions([
            {
                originX: 'center',
                originY: 'bottom',
                overlayX: 'center',
                overlayY: 'top',
                panelClass: 'sbb-tooltip-content-below',
            },
            {
                originX: 'end',
                originY: 'bottom',
                overlayX: 'end',
                overlayY: 'top',
                offsetX: 5,
                panelClass: ['sbb-tooltip-content-below', 'sbb-tooltip-content-left'],
            },
            {
                originX: 'start',
                originY: 'bottom',
                overlayX: 'start',
                overlayY: 'top',
                offsetX: -5,
                panelClass: ['sbb-tooltip-content-below', 'sbb-tooltip-content-right'],
            },
            {
                originX: 'center',
                originY: 'top',
                overlayX: 'center',
                overlayY: 'bottom',
                panelClass: 'sbb-tooltip-content-above',
            },
            {
                originX: 'end',
                originY: 'top',
                overlayX: 'end',
                overlayY: 'bottom',
                offsetX: 5,
                panelClass: ['sbb-tooltip-content-above', 'sbb-tooltip-content-left'],
            },
            {
                originX: 'start',
                originY: 'top',
                overlayX: 'start',
                overlayY: 'bottom',
                offsetX: -5,
                panelClass: ['sbb-tooltip-content-above', 'sbb-tooltip-content-right'],
            },
        ]);
    }
    /** Stream of clicks outside of the tooltip panel. */
    _getOutsideClickStream() {
        if (!this._document) {
            return of(null);
        }
        return merge(fromEvent(this._document, 'mousedown'), fromEvent(this._document, 'touchend')).pipe(filter((event) => {
            // If we're in the Shadow DOM, the event target will be the shadow root, so we have to
            // fall back to check the first element in the path of the click event.
            const clickTarget = (this._isInsideShadowRoot && event.composedPath ? event.composedPath()[0] : event.target);
            return (this.overlayAttached &&
                clickTarget !== this.tooltipTrigger.nativeElement &&
                !this.tooltipTrigger.nativeElement.contains(clickTarget) &&
                !!this.tooltipRef &&
                !this.tooltipRef.overlayElement.contains(clickTarget));
        }));
    }
    /**
     * This method listens to a stream of panel closing actions and resets the
     * stream every time the option list changes.
     */
    _subscribeToClosingActions() {
        const firstStable = this._zone.onStable.asObservable().pipe(take(1));
        // When the zone is stable initially, and when the option list changes...
        return (merge(firstStable)
            .pipe(
        // create a new stream of panelClosingActions, replacing any previous streams
        // that were created, and flatten it so our stream only emits closing events...
        switchMap(() => {
            return this.panelClosingActions;
        }), 
        // when the first closing event occurs...
        take(1))
            // set the value, close the panel, and complete.
            .subscribe(() => {
            this.close();
        }));
    }
    _createPopup() {
        const overlayConfig = new OverlayConfig({
            positionStrategy: this._createTooltipPositionStrategy(),
            hasBackdrop: false,
            scrollStrategy: this._scrollStrategy(),
            panelClass: this._panelClass,
        });
        // We want to resolve this once, as late as possible so that we can be
        // sure that the element has been moved into its final place in the DOM.
        if (this._isInsideShadowRoot == null) {
            this._isInsideShadowRoot = !!_getShadowRoot(this._elementRef.nativeElement);
        }
        this.tooltipRef = this._overlay.create(overlayConfig);
        this.tooltipRef.overlayElement.setAttribute('role', 'tooltip');
        // Use the `keydownEvents` in order to take advantage of
        // the overlay event targeting provided by the CDK overlay.
        this.tooltipRef.keydownEvents().subscribe((event) => {
            // Close when pressing ESCAPE, based on the a11y guidelines.
            // See: https://www.w3.org/TR/wai-aria-practices-1.1/#textbox-keyboard-interaction
            if (event.keyCode === ESCAPE) {
                this._closeKeyEventStream.next();
            }
        });
    }
}
SbbTooltipBase.decorators = [
    { type: Directive }
];
SbbTooltipBase.ctorParameters = () => [
    { type: Overlay },
    { type: SbbTooltipRegistryService },
    { type: undefined },
    { type: NgZone },
    { type: ChangeDetectorRef },
    { type: ElementRef },
    { type: undefined }
];
SbbTooltipBase.propDecorators = {
    icon: [{ type: Input }],
    _contentIcon: [{ type: ContentChild, args: [SbbIconDirective, { read: TemplateRef },] }],
    tooltipContentPortal: [{ type: ViewChild, args: ['tooltipTemplate', { static: true },] }],
    tooltipTrigger: [{ type: ViewChild, args: ['triggerButton', { static: true },] }],
    defaultIcon: [{ type: ViewChild, args: ['defaultIcon', { read: TemplateRef, static: true },] }],
    overlayWithPush: [{ type: Input }],
    overlayWithLockedPosition: [{ type: Input }],
    overlayViewportMargin: [{ type: Input }],
    opened: [{ type: Output }],
    closed: [{ type: Output }]
};

/**
 * Generated bundle index. Do not edit.
 */

export { SBB_TOOLTIP_SCROLL_STRATEGY, SBB_TOOLTIP_SCROLL_STRATEGY_FACTORY, SBB_TOOLTIP_SCROLL_STRATEGY_FACTORY_PROVIDER, SbbTooltipBase, SbbTooltipChangeEvent, SbbTooltipRegistryService };
//# sourceMappingURL=tooltip.js.map
