import { InjectionToken, Injectable, Inject, LOCALE_ID, Optional } from '@angular/core';
import { TitleCasePipe, DatePipe } from '@angular/common';

/** Adapts type `D` to be usable as a date by cdk-based components that work with dates. */
class SbbDateAdapter {
    /**
     * Attempts to deserialize a value to a valid date object. This is different from parsing in that
     * deserialize should only accept non-ambiguous, locale-independent formats (e.g. a ISO 8601
     * string). The default implementation does not allow any deserialization, it simply checks that
     * the given value is already a valid date object or null. The `<sbb-datepicker>` will call this
     * method on all of it's `@Input()` properties that accept dates. It is therefore possible to
     * support passing values from your backend directly to these properties by overriding this method
     * to also deserialize the format used by your backend.
     * @param value The value to be deserialized into a date object.
     * @returns The deserialized date object, either a valid date, null if the value can be
     *     deserialized into a null date (e.g. the empty string), or an invalid date.
     */
    deserialize(value) {
        if (value == null || (this.isDateInstance(value) && this.isValid(value))) {
            return value;
        }
        return this.invalid();
    }
    /**
     * Sets the locale used for all dates.
     * @param locale The new locale.
     */
    setLocale(locale) {
        this._locale = locale;
    }
    /**
     * Compares two dates.
     * @param first The first date to compare.
     * @param second The second date to compare.
     * @returns 0 if the dates are equal, a number less than 0 if the first date is earlier,
     *     a number greater than 0 if the first date is later.
     */
    compareDate(first, second) {
        return (this.getYear(first) - this.getYear(second) ||
            this.getMonth(first) - this.getMonth(second) ||
            this.getDate(first) - this.getDate(second));
    }
    /**
     * Checks if two dates are equal.
     * @param first The first date to check.
     * @param second The second date to check.
     * @returns Whether the two dates are equal.
     *     Null dates are considered equal to other null dates.
     */
    sameDate(first, second) {
        if (first && second) {
            const firstValid = this.isValid(first);
            const secondValid = this.isValid(second);
            if (firstValid && secondValid) {
                return !this.compareDate(first, second);
            }
            return firstValid === secondValid;
        }
        return first === second;
    }
    /**
     * Clamp the given date between min and max dates.
     * @param date The date to clamp.
     * @param min The minimum value to allow. If null or omitted no min is enforced.
     * @param max The maximum value to allow. If null or omitted no max is enforced.
     * @returns `min` if `date` is less than `min`, `max` if date is greater than `max`,
     *     otherwise `date`.
     */
    clampDate(date, min, max) {
        if (min && this.compareDate(date, min) < 0) {
            return min;
        }
        if (max && this.compareDate(date, max) > 0) {
            return max;
        }
        return date;
    }
}

const SBB_DATE_FORMATS = new InjectionToken('sbb-date-formats');

const SBB_DATE_PIPE_DATE_FORMATS = {
    dateInput: 'EEEEEE, dd.MM.yyyy',
    dateA11yLabel: 'dd MM yyyy',
};

const SBB_DATEPICKER_2DIGIT_YEAR_PIVOT = new InjectionToken('SBB_DATEPICKER_2DIGIT_YEAR_PIVOT');

/**
 * Matches strings that have the form of a valid RFC 3339 string
 * (https://tools.ietf.org/html/rfc3339). Note that the string may not actually be a valid date
 * because the regex will match strings an with out of bounds month, date, etc.
 */
const ISO_8601_REGEX = /^\d{4}-\d{2}-\d{2}(?:T\d{2}:\d{2}:\d{2}(?:\.\d+)?(?:Z|(?:(?:\+|-)\d{2}:\d{2}))?)?$/;
/** Creates an array and fills it with values. */
function range(length, valueFunction) {
    const valuesArray = Array(length);
    for (let i = 0; i < length; i++) {
        valuesArray[i] = valueFunction(i);
    }
    return valuesArray;
}
class SbbNativeDateAdapter extends SbbDateAdapter {
    constructor(_locale, yearPivot) {
        super();
        this._locale = _locale;
        this._titleCasePipe = new TitleCasePipe();
        this._datePipe = new DatePipe(_locale);
        this._yearPivot = typeof yearPivot === 'number' ? yearPivot : new Date().getFullYear() - 1975;
    }
    getYear(date) {
        return date.getFullYear();
    }
    getMonth(date) {
        return date.getMonth();
    }
    getDate(date) {
        return date.getDate();
    }
    getDayOfWeek(date) {
        return date.getDay();
    }
    getMonthName(date) {
        return this._datePipe.transform(date, 'LLLL');
    }
    getMonthNames(style) {
        let format;
        switch (style) {
            case 'long':
                format = 'LLLL';
                break;
            case 'narrow':
                format = 'LLLLL';
                break;
            default:
                format = 'LLL';
                break;
        }
        return range(12, (i) => this._datePipe.transform(new Date(2017, i, 1), format));
    }
    getDateNames() {
        return range(31, (i) => this._datePipe.transform(new Date(2017, 0, i + 1), 'd'));
    }
    getDayOfWeekNames(style) {
        let format;
        switch (style) {
            case 'long':
                format = 'EEEE';
                break;
            case 'narrow':
                format = 'EEEEE';
                break;
            default:
                format = 'EEE';
                break;
        }
        return range(7, (i) => this._datePipe.transform(new Date(2017, 0, i + 1), format));
    }
    getYearName(date) {
        return this._datePipe.transform(date, 'yyyy');
    }
    getFirstDayOfWeek() {
        return 1;
    }
    getNumDaysInMonth(date) {
        const lastDayOfMonth = new Date(0);
        lastDayOfMonth.setFullYear(date.getFullYear(), date.getMonth() + 1, 0);
        lastDayOfMonth.setHours(0, 0, 0, 0);
        return lastDayOfMonth.getDate();
    }
    clone(date) {
        return new Date(date.valueOf());
    }
    createDate(year, month, date) {
        // Check for invalid month and date (except upper bound on date which we have to check after
        // creating the Date).
        if (month < 0 || month > 11) {
            throw Error(`Invalid month index "${month}". Month index has to be between 0 and 11.`);
        }
        if (date < 1) {
            throw Error(`Invalid date "${date}". Date has to be greater than 0.`);
        }
        const result = this._createDateWithOverflow(year, month, date);
        // Check that the date wasn't above the upper bound for the month, causing the month to overflow
        if (result.getMonth() !== month) {
            throw Error(`Invalid date "${date}" for month with index "${month}".`);
        }
        return result;
    }
    today() {
        const date = new Date();
        date.setHours(0, 0, 0, 0);
        return date;
    }
    parse(value) {
        if (typeof value === 'number') {
            return new Date(value);
        }
        else if (typeof value === 'string') {
            return this._parseStringDate(value);
        }
        return null;
    }
    format(date, displayFormat) {
        if (!this.isValid(date)) {
            throw Error('NativeDateAdapter: Cannot format invalid date.');
        }
        return this._titleCasePipe.transform(this._datePipe.transform(date, displayFormat));
    }
    addCalendarYears(date, years) {
        return this.addCalendarMonths(date, years * 12);
    }
    addCalendarMonths(date, months) {
        const targetMonth = date.getMonth() + months;
        const dateWithCorrectMonth = new Date(0);
        dateWithCorrectMonth.setFullYear(date.getFullYear(), targetMonth, 1);
        dateWithCorrectMonth.setHours(0, 0, 0, 0);
        const daysInMonth = this.getNumDaysInMonth(dateWithCorrectMonth);
        const newDate = this.clone(date);
        // Adapt last day of month for shorter months
        newDate.setMonth(targetMonth, Math.min(daysInMonth, date.getDate()));
        return newDate;
    }
    addCalendarDays(date, days) {
        const newDate = this.clone(date);
        newDate.setDate(newDate.getDate() + days);
        return newDate;
    }
    toIso8601(date) {
        return this._datePipe.transform(date, 'yyyy-MM-dd');
    }
    /**
     * Returns the given value if given a valid Date or null. Deserializes valid ISO 8601 strings
     * (https://www.ietf.org/rfc/rfc3339.txt) into valid Dates and empty string into null. Returns an
     * invalid date for all other values.
     */
    deserialize(value) {
        if (typeof value === 'string') {
            if (!value) {
                return null;
            }
            // The `Date` constructor accepts formats other than ISO 8601, so we need to make sure the
            // string is the right format first.
            if (ISO_8601_REGEX.test(value)) {
                const date = new Date(value);
                if (this.isValid(date)) {
                    return date;
                }
            }
        }
        return super.deserialize(value);
    }
    isDateInstance(obj) {
        return obj instanceof Date;
    }
    isValid(date) {
        return !isNaN(date.valueOf());
    }
    invalid() {
        return new Date(NaN);
    }
    _parseStringDate(value) {
        const match = /^(\w+,[ ]?)?(\d+)\.(\d+)\.(\d+)$/.exec(value);
        if (!match) {
            return null;
        }
        const date = this._createDateWithOverflow(+match[4], +match[3] - 1, +match[2]);
        return this._normalizeYear(date);
    }
    _normalizeYear(date) {
        if (date.getFullYear() < 0 || date.getFullYear() >= 100) {
            return date;
        }
        const yearOffset = date.getFullYear() <= this._yearPivot ? 2000 : 1900;
        return new Date(date.getFullYear() + yearOffset, date.getMonth(), date.getDate());
    }
    /** Creates a date but allows the month and date to overflow. */
    _createDateWithOverflow(year, month, date) {
        const result = new Date(year, month, date);
        // We need to correct for the fact that JS native Date treats years in range [0, 99] as
        // abbreviations for 19xx.
        if (year >= 0 && year < 100) {
            result.setFullYear(this.getYear(result) - 1900);
        }
        return result;
    }
}
SbbNativeDateAdapter.decorators = [
    { type: Injectable }
];
SbbNativeDateAdapter.ctorParameters = () => [
    { type: String, decorators: [{ type: Inject, args: [LOCALE_ID,] }] },
    { type: Number, decorators: [{ type: Optional }, { type: Inject, args: [SBB_DATEPICKER_2DIGIT_YEAR_PIVOT,] }] }
];

class SbbBusinessDateAdapter extends SbbNativeDateAdapter {
    constructor(_locale, yearPivot) {
        super(_locale, yearPivot);
        this._locale = _locale;
    }
    _parseStringDate(value) {
        return super._parseStringDate(value) || this._parseStringDateBusiness(value);
    }
    _parseStringDateBusiness(value) {
        const match = /^(\w+,[ ]?)?(\d{2})(\d{2})(\d{2}|\d{4})$/.exec(value);
        if (!match) {
            return null;
        }
        return this._normalizeYear(this._createDateWithOverflow(+match[4], +match[3] - 1, +match[2]));
    }
}
SbbBusinessDateAdapter.decorators = [
    { type: Injectable }
];
SbbBusinessDateAdapter.ctorParameters = () => [
    { type: String, decorators: [{ type: Inject, args: [LOCALE_ID,] }] },
    { type: Number, decorators: [{ type: Optional }, { type: Inject, args: [SBB_DATEPICKER_2DIGIT_YEAR_PIVOT,] }] }
];

const SBB_BUSINESS_DATE_ADAPTER = {
    provide: SbbDateAdapter,
    useClass: SbbBusinessDateAdapter,
};

/**
 * Generated bundle index. Do not edit.
 */

export { SBB_BUSINESS_DATE_ADAPTER, SBB_DATEPICKER_2DIGIT_YEAR_PIVOT, SBB_DATE_FORMATS, SBB_DATE_PIPE_DATE_FORMATS, SbbBusinessDateAdapter, SbbDateAdapter, SbbNativeDateAdapter };
//# sourceMappingURL=datetime.js.map
